package detail

import (
	log "dbackup/logging"
	"fmt"
	"github.com/robfig/cron/v3"
	"io/ioutil"
	"os"
	"os/exec"
	"runtime"
	"strconv"
	"strings"
	"time"
)

const (
	PGDumpCmd = "pg_dump"
)

type PGConfig struct {
	PgHome      string   `toml:"pg_home"`
	Host        string   `toml:"host"`
	Port        int      `toml:"port"`
	DB          []string `toml:"db"`
	Username    string   `toml:"username"`
	Password    string   `toml:"password"`
	Destination string   `toml:"destination"`
	Keep        int64    `toml:"keep"`
	Cron        string   `toml:"cron"`
	Options     []string `toml:"options"`
}

type Postgres struct {
	PGConfig []PGConfig
}

func (x *Postgres) Setup() {
	for _, c := range x.PGConfig {
		if strings.Index(c.Destination, "\\") != -1 {
			log.Fatal("invalid destination: %s, must use Unix Path Seperator `/`")
		}
		for _, i := range c.DB {
			if e := os.MkdirAll(c.Destination+"/"+i, os.ModePerm); e != nil {
				log.Fatal("can't mkdir: %s, err: %s", c.Destination, e)
			}
		}
	}
}

func (x *Postgres) AddTask(t *Task) error {
	x.Setup()
	log.Info("postgres adding task, count: %d", len(x.PGConfig))
	for _, cfg := range x.PGConfig {
		if e := cfg.AddCron(t.Cron); e != nil {
			return e
		}
	}
	return nil
}

func (x *PGConfig) Run() {
	log.Info("backup postgresql...")
	for _, db := range x.DB {
		path := fmt.Sprintf(`%s/%s/%v_%v.dump`, x.Destination, db, db, time.Now().Format("20060102-150405"))
		options := append(x.GetOptions(db), "-f", path) //fmt.Sprintf(`-f%v`, path))
		cmd := fmt.Sprintf("%s %s", PGDumpCmd, strings.Join(options, " "))
		log.Info("running task: `%s`", cmd)
		if runtime.GOOS == "windows" {
			cmd = fmt.Sprintf(`set PGPASSWORD=%s&& %s`, x.Password, cmd)
			mo := exec.Command("cmd", "/C", cmd)
			mo.Env = []string{fmt.Sprintf(`PATH=%s`, x.PgHome)}
			out, err := mo.CombinedOutput()
			if err != nil {
				log.Fatal("failed: %s, %s", err, out)
			} else {
				log.Info("ok")
				x.CleanUp(db)
			}
		} else {
			cmd = fmt.Sprintf(`PGPASSWORD="%s" %s`, x.Password, cmd)
			mo := exec.Command("bash", "-c", cmd)
			mo.Env = []string{fmt.Sprintf("PATH=%s:%s", os.Getenv("PATH"), x.PgHome)}
			out, err := mo.CombinedOutput()

			if err != nil {
				log.Fatal("failed: %s, %s", err, out)
			} else {
				log.Info("ok")
				x.CleanUp(db)
			}
		}
	}
}

func (x *PGConfig) CleanUp(db string) {
	if x.Keep < 0 {
		return
	}
	root := fmt.Sprintf("%s/%s", x.Destination, db)
	info, e := ioutil.ReadDir(root)
	if e != nil {
		log.Fatal("can't list dir: %s, err: %s", x.Destination, e)
	}

	for _, i := range info {
		if strings.Index(i.Name(), db) == 0 {
			f := fmt.Sprintf("%s/%s", root, i.Name())
			o, e := os.Stat(f)
			if e != nil {
				log.Error("can't get last modify time for: %s, err: %s", f, e)
				continue
			}
			if time.Since(o.ModTime()) >= time.Duration(x.Keep*int64(time.Hour)) {
				log.Info("remove expired backup: %s", i.Name())
				_ = os.Remove(f)
			}
		}
	}
}

func (x *PGConfig) AddCron(c *cron.Cron) error {
	s := strings.Split(x.Cron, " ")
	i := 0
	for j := 0; j < len(s); j++ {
		if len(strings.TrimSpace(s[j])) != 0 {
			s[i] = s[j]
			i += 1
		}
	}
	s = s[0:i]
	log.Info("%s", s)
	if len(s) != 5 {
		return fmt.Errorf("invalid cron format: %s", x.Cron)
	}
	_, e := c.AddJob(strings.Join(s, " "), x)
	return e
}

func (x *PGConfig) GetOptions(db string) []string {
	options := x.Options

	options = append(options, "-d", db, "-h", x.Host, "-p", strconv.Itoa(x.Port), "-U", x.Username)

	return options
}
